/*!
 * Ombutel
 * plugins needed: bootstrap, jquery validation, jquery form, jquery ui, jquery ui touch and all extended from those
 */

// polyfill for Array.range
if (![].range) {
	Array.prototype.range = function(start, end) {
		var result = [];
		for (var i = start; i <= end; ++i) { result.push(i); }
		return result;
	};
}

if (!("contains" in String.prototype)) {
	String.prototype.contains = function(needle) {
		return this.indexOf(needle) != -1;
	}
}

// Fill dropdown with JSON data
jQuery.fn.fill = function(data, callback, prepend) {
	var element = this;
	element.empty();
	if (prepend) {
		var option = $("<option/>");
		option.val(prepend[0]);
		option.text(prepend[1]);
		element.append(option);
	}
	if (data) {
		$.each(data, function() {
			var option = $("<option/>");
			callback.apply(option, [ this ]);
			element.append(option);
		});
	}
	element.prop("selectedIndex", 0);
	return this;
};

var Validations = {
	ignore: ".ignore, :hidden",
	errorClass: "validations-tooltip",
	showErrors: function (errorMap, errorList) {
		$.each(this.validElements(), function (index, element) {
			var formElement = $(element);
			formElement.data("title", "");
			if (formElement.parent('.input-group').length) {
				var input_group = formElement.parent('.input-group');
				input_group.parent('.form-group').removeClass('has-error');
			} else {
				formElement.parent('.form-group').removeClass('has-error');
			}

			formElement.tooltip("destroy");
		});

		$.each(errorList, function (index, error) {
			var formElement = $(error.element);
			formElement.tooltip("destroy")
				.data("title", error.message);

			if (formElement.parent('.input-group').length) {
				var input_group = formElement.parent('.input-group');
				input_group.parent('.form-group').addClass('has-error');
			} else {
				formElement.parent('.form-group').addClass('has-error');
			}
			formElement.tooltip({
				trigger: 'hover focus click focusout',
				template: '<div class="tooltip validations-tooltip" role="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>',
				content: function () {
					return $(this).prop('title');
				}
			});
		});
	}
};

var repeatableSettings = {
	wrapper: '.repeat-wrapper',
	container: '.repeat-container',
	row: '.repeat-row',
	add: '.repeat-add',
	remove: '.repeat-remove',
	move: '.repeat-move',
	template: '.repeat-template'
};

var loginForm = $('#pbxlogin-form');

var bodyWidth;
var bodyHeight;
var docBody = $("body");
var spinner = $("#spinner-section");
var mainTabs = $("#ombutel-main-tabs");
var modulesHeaderContent = $("#modules-content-header");
var modulesContent = $("#ombutel-modules-content");
var globSearchInput = $("#ombutel-search");
var Ombutel = $(document);
var isMultiTab;
var current_tab_name;

Ombutel.ready(OmbutelInit);

$(window).on('load resize', function () {
	bodyWidth = docBody.width();
	bodyHeight = Ombutel.height();
	spinner.fadeOut();
	spinner.remove();
	tooltipPBX();
	friendlyActionButtons();
	updateTabControls();
	mobileInteraction();
});

function OmbutelInit() {
	if (loginForm.length) {
		loginForm.validate(Validations);
		var user = loginForm.find('#userid');
		var pass = loginForm.find('#admin_pass');
		if(user.length) {
			user.focus();
		} else if(pass.length) {
			pass.focus();
		}
	}
	bodyHeight = Ombutel.height();
	$(document).on("submit", "form", function() {
		if(!$(this).data('raw')) {
			$(this).ajaxSubmit({
				success: processServerResponse,
				dataType: "json",
				url: $(this).attr("action"),
				resetForm: false,
				error: processServerErrors,
				data: { baseurl: window.location.protocol + "//" + window.location.host },
			});
			return false;
		}
	});

	Ripple();

	register_tab_events();

	isMultiTab = !!(multi_tab === 'yes' || multi_tab === '1');
	noMultiTab();

	globSearchInput.keypress(function (e) {
		if (e.which == 13) {
			globalSearch($(this).val());
		}
	});

	$("#fire-search").click(function() {
		globalSearch(globSearchInput.val());
	});

	$("#right-back-drop").click(hideRightMenu);
	$("#aside-nav-right #close-module-menu").click(hideRightMenu);

	tooltipPBX();

	registerMobileMainTabsEvents();

	tabsContextMenu();

	showStartUp();

	var rv;
	if (rv = isIE()) {
		$("body,html").addClass("ie-" + rv);
	}

	$("#apply-asterisk-changes").on('click',function(){
		check_dahdi();
	});
}

function isIE() {
	var rv = -1;
	if (navigator.appName == 'Microsoft Internet Explorer') {
		var ua = navigator.userAgent;
		var re  = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
		if (re.exec(ua) != null)
			rv = parseFloat( RegExp.$1 );
	}
	else if (navigator.appName == 'Netscape') {
		var ua = navigator.userAgent;
		var re  = new RegExp("Trident/.*rv:([0-9]{1,}[\.0-9]{0,})");
		if (re.exec(ua) != null)
			rv = parseFloat( RegExp.$1 );
	}

	return rv == -1 ? false : rv;
}

function noMultiTab() {
	if (!isMultiTab) {
		modulesHeaderContent.removeClass('multi-tab').addClass('no-multi-tab').html('');
		modulesContent.removeClass('tab-content').html('');
	}
}

function showStartUp(){
	if(typeof startup != "undefined" && startup != ''){
		pbx_request(startup);
	}
}

function globalSearch(criterion) {
	if(criterion === null || criterion === ''){
		sendNotification('warning','Nothing to search');
		return;
	}

	globSearchInput.val('');
	pbx_request('search','getContent','get',criterion,'Search',true);
}

function pbx_request(item, method, mode, data, label, showTab) {
	var Obj = item;
	var ObjMethod = (method == null ? "getContent" : method);
	var postMode = (mode == null ? "add" : mode);
	var postData = (data == null ? 0 : data);

	return $.ajax({
		type: "POST",
		url: "/",
		dataType: 'json',
		data: {
			class: Obj,
			method: ObjMethod,
			mode: postMode,
			data: postData,
			baseurl: window.location.protocol + "//" + window.location.host,
		},
		success: function (Response) {
			processServerResponse(Response);
			delegatePBX();

		},
		complete: function() {
			request_running = false;
		},
		error: processServerErrors
	});
}

function addModuleHeaderTitle(new_tab_name, title) {
	if (!modulesHeaderContent.length) {
		sendNotification('error', '[HM7790] You did not define the Header Module Section', 2000);
		return false;
	}

	modulesHeaderContent.html('<h3 class="no-margin"> '+ title +' </h3>');
	modulesContent.removeClass(current_tab_name);
	modulesContent.addClass(new_tab_name);
	current_tab_name = new_tab_name;
}

function addPBXTab(contentID, label) {
	//Close Welcome tabs, this is not necessary
	var welcomeTab = mainTabs.find('a[href="#welcome"]');
	if(welcomeTab.length){
		welcomeTab.parent().remove();
	}


	if (!mainTabs.length && !modulesContent.length) {
		sendNotification('error', '[T7890] You did not define the Tab Section', 2000);
		return false;
	}

	var contentTab = $("#" + contentID);

	if (!contentTab.length) {
		mainTabs.prepend('<li role="presentation">' +
		'<a href="#' + contentID + '" aria-controls="' + contentID + '" role="tab" data-toggle="tab"> ' + label + ' ' +
		'<i class="fa fa-close fa-fw fa-2x close-tab"></i>' +
		'<i class="fa fa-refresh fa-fw fa-2x reload-tab"></i>' +
		' </a> ' +
		'</li>');

		modulesContent.prepend('<div role="tabpanel" class="tab-pane fade" id="' + contentID + '"' + '> ' + label + ' </div>');
	} else {
		var tab = mainTabs.find('a[href="#' + contentID + '"]');
		var cloneTab = tab.parent('li').clone();
		tab.parent().remove();
		mainTabs.prepend(cloneTab);

	}

	mainTabs.find('a[href="#' + contentID + '"]').tab('show');
	updateTabControls();
	register_tab_events();
}

var request_running = false;
function register_tab_events() {
	$(".close-tab").click(function () {
		var tabContentId = $(this).parent('a').attr("href");
		var module = tabContentId.substr(5);
		if (ombu.timers[module]) {
			for (var name in ombu.timers[module]) {
				clearInterval(ombu.timers[module][name]);
			}
		}
		$(this).parent('a').parent('li').remove();
		$(tabContentId).remove();
		updateTabControls();
		showMainFirstTab();
	});

	$(".reload-tab").click(function () {
		if (request_running) { // ajax is still running. Don't do anything
			return;
		}
		var tabContentId = $(this).parent('a').attr("href");
		var module_name = $('div' + tabContentId).find("input#class");
		var data = $('div' + tabContentId).find("input.tab-content");
		if(data.length && module_name.length) {
			request_running = true;
			pbx_request(module_name.val(), 'getContent', 'edit', data.val());
		} else if(module_name.length) {
			request_running = true;
			pbx_request(module_name.val());
		}
	});
}

function showMainFirstTab(){
	var tabToShow = mainTabs.find('a:first');
	if(tabToShow.length === 0) return;
	tabToShow.parent('li').show();
	tabToShow.tab('show');
}

function tabsContextMenu(){
	$("#ombutel-main-tabs li").contextMenu({
		menuSelector: "#tab-context-menu",
		menuSelected: function (invokedOn, selectedMenu) {
			var action = selectedMenu.attr('id');
			var tag = invokedOn.prop('tagName');
			var contentId = null;
			var liElem = null;

			if(tag === 'A'){
				contentId = invokedOn.attr('href');
				liElem = invokedOn.parent('li');
			}else if(tag === 'I'){
				contentId = invokedOn.parent('a').attr('href');
				liElem = invokedOn.parent('a').parent();
			}else if(tag === 'LI'){
				contentId = invokedOn.find('a').attr('href');
				liElem = invokedOn;
			}else{
				return false;
			}

			var prev = liElem.prevAll('li');
			var next = liElem.nextAll('li');

			switch (action){
				case 'closeSelf':
					liElem.remove();
					$(contentId).remove();
					updateTabControls();
					showMainFirstTab();
					break;
				case 'closeOthers':
					prev.each(function(){
						var prevContentId = $(this).find('a').attr('href');
						$(this).remove();
						$(prevContentId).remove();
					});

					next.each(function(){
						var nextContentId = $(this).find('a').attr('href');
						$(this).remove();
						$(nextContentId).remove();
					});

					updateTabControls();
					showMainFirstTab();
					break;

				case 'closeRightTabs':
					next.each(function(){
						var nextContentId = $(this).find('a').attr('href');
						$(this).remove();
						$(nextContentId).remove();
					});

					updateTabControls();
					break;
				case 'reload': {
					var module_name = $('div' + contentId).find("input#class");
					if(module_name.length) {
						pbx_request(module_name.val());
					}
					break;
				}
				default:
					sendNotification('warning', 'Unhandled tab context menu action "' + action +'"');
					break;
			}
		}
	});
}


function registerMobileMainTabsEvents() {
	$("#tab-control-left").click(function() {
		var active = mainTabs.find('> .active');
		var prev = active.prev('li').find('a');
		if (prev.length) {
			prev.tab('show');
		} else {
			$(this).hide();
		}

		updateTabControls();
	});

	$("#tab-control-right").click(function() {
		var active = mainTabs.find('> .active');
		var next = active.next('li').find('a');
		if (next.length) {
			next.tab('show');
		} else {
			$(this).hide();
		}

		updateTabControls();

	});
}

function updateTabControls() {
	var controlLeft = $("#tab-control-left");
	var controlRight = $("#tab-control-right");
	var active = mainTabs.find('> .active');
	var prev = active.prev('li').find('a');
	var next = active.next('li').find('a');

	active.show();

	if (prev.length) {
		if (bodyWidth <= 480) {
			prev.parent('li').hide();
			controlLeft.show();
		} else {
			controlLeft.hide();
			prev.parent('li').show();
		}

	} else {
		controlLeft.hide();
	}

	if (next.length) {
		if (bodyWidth <= 480) {
			next.parent('li').hide();
			controlRight.show();
		} else {
			next.parent('li').show();
			controlRight.hide();
		}

	} else {
		controlRight.hide();
	}

}

function processServerResponse(data) {
	RemoveDialog('modal-module-delete');
	if (typeof data !== 'undefined' && data !== null) {
		if (data.trigger) {
			$(data.trigger.target).trigger(data.trigger.type, data.trigger.args);
		}
		switch (data.state) {
			case 'size_exceed': {
				exceed_size(data.notification.title, data.notification.text);
				break;
			}
			case 'success':
				if (typeof data.notification !== 'undefined') {
					sendNotification('success', data.notification.text);
				}

				if (typeof data.html !== 'undefined') {
					if (typeof data.action !== 'undefined') {
						processReceivedData(data);
					} else {
						sendNotification('warning', 'No action set for html received');
					}
				}

				if (data.triggers) {
					for (const trigger of data.triggers) {
						$(trigger.target).trigger(trigger.type, trigger.args);
					}
				}

				break;

			case 'error':
				if (typeof data.notification !== 'undefined') {
					sendNotification('error', data.notification.text, 5000);
				} else {
					sendNotification('error', 'Unexpected error received', 5000);
				}
				break;

			case 'message':
				sendNotification(data.notification.type, data.notification.text, data.notification.time);
				if(typeof data.notification.reload !== 'undefined' && data.notification.reload === true){
					showReloadButton();
				}
				break;

			case 'exception':
				showException(data.exception, data.trace);
				break;

			default:
				sendNotification('error', 'Not handled state received');
				break;
		}
	} else {
		sendNotification('error', 'Not valid data received');
	}
}

function processReceivedData(data) {
	switch (data.action) {
		case "popup":
			window.open(data.url, "_blank");
			break;
		case 'dialog':
			removeModalBackDrop();
			ShowDialog(data.dialog, data.html, data.script);
			break;
		case 'remove-dialog':
			removeModalBackDrop();
			RemoveDialog(data.dialog);
			break;
		case 'reload':
			ReloadPage(data);
			break;
		case 'puthtml':
			removeModalBackDrop();
			PutHtml(data.html, data.elem);
			break;
		case 'resetform':
			ResetForm(data.form);
			break;
		case 'pbx':
			removeModalBackDrop();
			PBX(data);
			break;
		case 'pbxmenu':
			pbxMenu(data.dataItem);
			break;
		case 'sysreload-applied':
			removeModalBackDrop();
			systemReloadApplied();
			break;
	}
}

function ShowDialog(dialog, html, script) {
	$("#" + dialog).remove();
	$("body").append(html);
	const modal = $("#" + dialog);
	if (script !== undefined) {
		modal.on("show.bs.modal", new Function(script));
	}
	modal.modal('show');
}

function PutHtml(html, elem) {
	var elemId = $("#" + elem);
	elemId.html('');
	elemId.html(html);

	delegatePBX();
}

function RemoveDialog(dialog) {
	var dialogModal = $("#" + dialog);
	dialogModal.modal('hide');
	removeModalBackDrop();
	dialogModal.remove();
}

function ReloadPage(data) {
	if (typeof data.time !== 'undefined') {
		setInterval(location.reload(), data.time)
	} else {
		location.reload();
	}

}

function ResetForm(form) {
	$("#" + form).reset();
}

function PBX(data) {
	if (data.menu == "hide") {
		$("#aside-nav-right ul").empty();
		hideRightMenu();
	}
	if (data.html !== undefined && data.html != "") {
		if (!isMultiTab) {
			for (i in ombu.timers) {
				for (j in ombu.timers[i]) {
					clearInterval(ombu.timers[i][j]);
				}
			}
			ombu.timers = [];
			addModuleHeaderTitle(data.elem, data.label);
		} else {
			addPBXTab(data.elem, data.label);
		}
		var container = (isMultiTab) ? $("#" + data.elem) : modulesContent;
		container.html(data.html);
		container.data("menu_title", data.menu_title);
		container.data("module", data.module);
	}
	var promises = [];
	if (data.js !== undefined) {
		var xhrs = [];
		data.js.forEach(function (url) {
			xhrs.push($.ajax({ url: url, dataType: "text" }));
		});
		promises.push(new Promise(function(resolve, reject) {
			Promise.all(xhrs).then(function(scripts) {
				scripts.forEach($.globalEval);
				resolve();
			});
		}));
	}
	if (data.css !== undefined) {
		var cache = jQuery.ajaxSetup({}).cache;
		data.css.forEach(function (url) {
			if (cache !== true) { url += "?=" + Date.now(); }
			var link = $("<link>").attr("rel", "stylesheet").attr("href", url);
			promises.push(new Promise(function(resolve, reject) {
				link.on("load", resolve);
				link.on("error", reject);
			}));
			$("head").append(link);
		});
	}

	//This function must be exist for each theme
	mobileInteraction();

	$(".custom-modal").hide();
	if (data.reload) {
		showReloadButton();
	}

	delegatePBX();

	Promise.all(promises).then(function() {
		document.dispatchEvent(new CustomEvent("tab-loaded", { detail: container }));
	});
}

function showReloadButton(){
	var btnApplyChanges = $("#apply-asterisk-changes");
	btnApplyChanges.removeClass('hidden');
	btnApplyChanges.addClass('animated shake-animation');

	btnApplyChanges.one('webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function(){
		btnApplyChanges.removeClass('animated shake-animation');
	});
}

function systemReloadApplied(){
	$("#apply-asterisk-changes").addClass('hidden');
}

function pbxMenu(menuItems) {
	$("#module-menu-items").empty().removeClass("spinning");

	if (menuItems.hasOwnProperty("items")) {
		var menu = menuItems.items;
		$.each(menu, function (index, item) {
			var params = 'onclick="javascript:pbx_request(\'' + menuItems.module + '\',\'getContent\',\'edit\',\'' + item.id + '\')"';

			 if(item.hasOwnProperty('action')) {
                var params = 'onclick="javascript:do_action(\'/' + item.action + '/' +item.id +'\')"';
            }

			if (item.hasOwnProperty("cust")) {
				params = item.cust;
			}

			var description = '', liClass = '';
			if ((typeof item.description === 'undefined') || (item.description === '' || item.description === null)) {
				liClass = 'no-desc';
                description = item.content;
			} else {
				description = item.description;
			}
			var image = (item.img !== undefined) ? item.img : menuItems.defaultimg;
            var title = (item.description ? item.description : description);
			var output = '<li class="' + liClass + ' toltip" data-toggle="popover" data-trigger="hover" title="' + title + '"> <a ' + params + '> ' +
				'<img src="' + image + '"/>' +
				' <div class="clear">' +
				' <span class="title"> ' + description + ' </span>' +
				' <span class="description"> ' + item.content + ' </span>' +
				' </div>' +
				'</a> </li>';

			$("#module-menu-items").append($("<ul>").append(output));
		});

		$("#aside-nav-right #filter-search-input").fastLiveFilter('#module-menu-items > ul', {
			timeout: 200,
			callback: function() {
				$("#module-menu-items")[0].scrollTop = 0;
			}
		});
	}
	document.dispatchEvent(new CustomEvent("list-loaded", { detail: $("#aside-nav-right") }));
	return true;
}

function sendRequest(functionToCall, data) {
	console.log(functionToCall);
	$.ajax({
		dataType: 'json',
		type: "POST",
		timeout: 60000,
		url: "/",
		data: {call: functionToCall, data: data},
		success: processServerResponse,
		error: processServerErrors
	});

	return false;
}

function do_action(url, params = {}) {
	return $.ajax({
		timeout: (url == "/apply-changes") ? 0 : 60000,
		dataType: "json",
		type: "GET",
		...params,
		url,
		success: function(response) {
			processServerResponse(response);
			delegatePBX();
		},
		error: processServerErrors,
		complete: function() {
			request_running = false;
		},
	});
}

function sendNotification(type, message, timeout) {
	return toastr[type](message, "", { timeOut: timeout });
}

function Ripple() {
	$(".ripple-me").click(function (evt) {
		var elem = $(this);
		$(".ripple").remove();
		elem.prepend("<span class='ripple'></span>");

		var display = elem.css("display");
		(display == 'inline' || display == 'inline-block') ? elem.css({display: 'inline-block'}) : elem.css({display: 'block'});
		var ripple = elem.find('.ripple');
		ripple.addClass('ripple-def-color');

		if ($(this).is('[ripple-color]')) {
			var rippleColor = this.getAttribute("ripple-color");
			if (rippleColor != "") {
				ripple.removeClass('ripple-def-color');
				ripple.addClass(rippleColor);
			}
		}

		ripple.removeClass('animate');
		if (!ripple.width() && !ripple.height()) {
			var max = Math.max(elem.outerWidth(), elem.outerHeight());
			ripple.css({width: max + 'px', height: max + 'px'});
		}

		var x = evt.pageX - elem.offset().left - ripple.width() / 2;
		var y = evt.pageY - elem.offset().top - ripple.height() / 2;
		ripple.css({top: y + 'px', left: x + 'px'});
		setTimeout(function () {
			ripple.addClass('animate');
		}, 1);
	});
}

function hideRightMenu() {
	$("#right-back-drop").hide();
	$("#aside-nav-right input #filter-search-input").val("");
	$("#aside-nav-right").animate({"right": "-300px"}, {
		duration: 200,
		complete: function() {
			$("#aside-nav-right").removeClass("active").trigger("transitionend");

		},
	});
}

function showRightMenu() {
	$("#right-back-drop").show();
	$("#module-menu-items").empty().addClass("spinning");
	const spinner = $("<div>").addClass("sk-spinner sk-spinner-circle");
	for (let index = 1; index <= 12; ++index ) {
		spinner.append($("<div>").addClass(`sk-circle${index} sk-circle`));
	}
	$("#module-menu-items").append(spinner);
	$("#aside-nav-right").animate({"right":"0"}, {
		duration: 200,
		start: function() { $("#aside-nav-right").addClass("active"); },
		complete: function() {
			$("#aside-nav-right #filter-search-input").focus();
			$("#aside-nav-right").trigger("transitionend");
		},
	});
	$("#aside-nav-right #filter-search-input").val("");
}

function delegatePBX() {
	var tabList = $(".tabs_list");
	var repeatable = $(".repeat");
	var bootstrapToggle = $(".chk_new");
	var multiSelect = $(".multiselect");
	var tPicker = $(".jui-timepicker, .timepicker");
	var dPicker = $('.datepicker');
	var comboMultiple = $('.combo-multiple');
	var comboTags = $('.combo-tags');

	if (tabList.length) {
		tabList.on("click", "a", function (e) {
			e.preventDefault();
			$(this).tab('show');
		});
	}

	if (repeatable.length) {
		repeatable.repeatable_fields(repeatableSettings);
	}

	if (bootstrapToggle.length) {
		bootstrapToggle.bootstrapToggle({
			onstyle: 'success',
			width: 70
		});
	}

	if (multiSelect.length) {
		multiSelect.multiselect({
			'sortable': 'both'
		});
	}

	if (tPicker.length) {
		tPicker.timepicker().removeClass('jui-timepicker timepicker');
	}

	if (dPicker.length) {
		dPicker.datepicker();
	}

	if (comboMultiple.length) {
		comboMultiple.select2({
			width: '100%'
		});
	}

	if (comboTags.length) {
		comboTags.select2({
			tags: true,
			width: '100%',
			tokenSeparators: [',', ' ']
		});
	}

	helpPopover();

	friendlyActionButtons();

	tooltipPBX();

	if (typeof themeDelegate !== 'undefined' && $.isFunction(themeDelegate)) {
		themeDelegate();
	}

	if(mainTabs.length){
		tabsContextMenu();
	}
}

function processServerErrors(xhr) {
	RemoveDialog('modal-module-delete');
	if (xhr.status == 0) {
		display_error(ombu.i18n.ombutel.connection_aborted);
	} else if (xhr.status >= 200 & xhr.status <= 299) {
		if (xhr.responseText == undefined || xhr.responseText == "") {
			display_error(ombu.i18n.ombutel.empty_server_response);
		} else {
			display_error([
				$("<p>").text(ombu.i18n.ombutel.server_error),
				$("<pre>").text(`${xhr.status} ${xhr.statusText}`),
			]);
		}
	} else {
		display_error(xhr.statusText);
	}
}

function showException(what, trace) {
	var content = [ $('<p>'+what+'</p><p>Traceback:</p>') ];
	for (i = 0; i < trace.length; ++i) {
		var frame = trace[i];
		var where = ('file' in frame) ? frame.file+'('+frame.line+')' : '[internal function]';
		var call = frame['function'];
		if ('class' in frame) {
			call = frame['class']+frame.type+frame['function'];
		}
		content.push($('<div>#'+i+' '+where+': '+call+'</div>'));
	}
	display_error(content);
}

function display_error(content) {
	const header = $("<div class=\"modal-header\">").append(
		$("<button type=\"button\" class=\"close\" data-dismiss=\"modal\">").append(
			$("<span aria-hidden=\"true\">").text("×"),
			$("<span class=\"sr-only\">").text(ombu.i18n.ombutel.close)),
		$("<div class=\"modal_header\">").append($("<h4 class=\"modal-title\">").text(ombu.i18n.ombutel.error)));
	const body = $("<div class=\"modal-body\">").append(content);
	const footer = $("<div class=\"modal-footer\">").append(
		$("<button type=\"button\" class=\"close-btn btn btn-default\" data-dismiss=\"modal\">").text(ombu.i18n.ombutel.close));
	const modal = $("<div class=\"modal fade error-modal-handle in\" tabindex=\"-1\" role=\"dialog\" aria-labelledby=\"modal-title\" aria-hidden=\"true\" data-backdrop=\"true\" data-keyboard=\"true\">").append(
		$("<div class=\"modal-dialog modal-lg\">").append(
			$("<div class=\"custom-modal-content modal-content\">").append(header, body, footer)));
	$(document.body).append(modal);
	modal.modal("show").on("hidden.bs.modal", function() {
		modal.remove();
	});
}

function multiple_select(combo, formElem, formTab, separator) {
	var selectedValues;

	separator = (separator === undefined || separator === null ? '\n' : separator);
	var selectedItems = $("#" + combo + " :selected");
	var formElement = (isMultiTab ? $("#" + formTab + " #" + formElem) : $("#" + formElem));

	switch (separator) {
		case "\n":
			selectedValues = "";

			selectedItems.each(function () {
				selectedValues += $(this).val() + "\n";
			});

			formElement.val(selectedValues);
			break;

		case ",":
			selectedValues = [];
			selectedItems.each(function () {
				selectedValues.push($(this).val());
			});

			formElement.val(selectedValues);
			break;

		default:
			break;
	}
}

function tooltipPBX() {
	$(".tooltip-pbx").tooltip({
		content: function () {
			return $(this).prop('title');
		}
	});
}

function removeModalBackDrop(){
	$('.modal-backdrop').remove();
}

function helpPopover(){
	var helpPopOver = $('.help-popover');

	if(helpPopOver.length){
		helpPopOver.popover({
			html:true,
			container: 'body',
			placement: 'auto top',
			content: function() {
				return $(this).attr('title').html();
			},
			delay: {
				"show": 100,
				"hide": 100
			}
		});
	}
}

function friendlyActionButtons(){
	var formActions = $('.frm-actions');
	if(formActions.length < 0){
		console.log(' -> Do not have actions this form');
		return;
	}


	if(bodyWidth <= 680){
		formActions.each(function($i, $e){
			var li = [];
			$(this).find('.btn').each(function($i, $e){
				if($(this).hasClass('frm-btn-action') || $(this).hasClass('mobile-action-list-btn')){
					return true;
				}

				var liItem = $("<li></li>").html($(this).clone().addClass('mobile-action-list-btn').removeClass('hidden'));
				$(this).addClass('hidden');
				li.push(liItem[0]);
			});

			//Create the new button
			var btngroup = $("<div></div>", {"class": "mobile-action-button btn-group dropup"});
			var button = $("<button></button>", {
				"class": "btn frm-btn-action btn-sm dropdown-toggle",
				"type": "button",
				"data-toggle": "dropdown",
				"aria-haspopup": "true",
				"aria-expanded": "false"
			}).html("<i class='fa fa-ellipsis-v fa-fw'></i> " + ombu.i18n.ombutel.actions);
			var ul = $("<ul></ul>",{"class": "dropdown-menu dropdown-menu-right mobile-actions-list"});
			ul.html(li);
			btngroup.html(button);
			btngroup.append(ul);

			var mobileButton = $(this).find('.mobile-action-button');
			if(mobileButton.length){
				mobileButton.remove();
			}

			$(this).append(btngroup);
		});
	}else{
       formActions.each(function($i, $e){
		   var mobileButton = $(this).find('.mobile-action-button');
		   if(mobileButton.length){
			   mobileButton.remove();
		   }
		   $(this).find('.btn').each(function($i, $e){
			   $(this).removeClass('hidden');
		   });
	   });
	}
}

function create_modal(title, accept) {
	var button = $("<button>").attr({"type" : "button", "data-dismiss" : "modal"});
	var modal = $("<div>").addClass("modal fade").attr("role", "dialog").append(
		$("<div>").addClass("modal-dialog").append(
			$("<div>").addClass("modal-content").append(
				$("<div>").addClass("modal-header").append(
					button.clone().addClass("sound_player_close close").append($("<span>").addClass('sound_player_close').html("&times;")),
					$("<h4>").addClass("modal-title").html(title)
				),
				$("<div>").addClass("modal-body"),
				$("<div>").addClass("modal-footer").append(
					button.clone().addClass("sound_player_close btn btn-default").html(ombu.i18n.ombutel.cancel),
					button.clone().addClass("btn btn-success").html(ombu.i18n.ombutel.accept)
				)
			)
		)
	);
	modal.on("hidden.bs.modal", function() { modal.remove(); });
	modal.on("click", ".btn-success", function() { accept.apply(modal); });
	$("body").append(modal);
	return modal;
}

function multiselect_modal(title, element, accept) {
	var multiselect = create_modal("Select " + title, accept);
	var copy = element.clone().removeAttr("id name");
	copy.val(element.val()); // Workaround
	multiselect.find(".modal-body").append(copy);
	copy.multiselect({'sortable': 'both'});
	multiselect.modal();
}

function live_search(search_field, elements) {
	var customDiv = $("<ul/>").addClass('livesearch');
	for (var key in elements) {
		customDiv.append($("<li>").html(elements[key].label).data('search_data', elements[key].data));
	}
	$(search_field).closest('div').append(customDiv);
	$('ul.livesearch li').bind('click', function() {
		$(search_field).val($(this).data('search_data'));
	});
}

function close_livesearch_list() {
	$('ul.livesearch').remove();
}

document.addEventListener("click", function (e) { // remove live search results
	close_livesearch_list();
});

function pagedtable(element) {
	this.data = {};
	this.element = $(element);
	this.element.prop("instance", this);
	this.page = this.element.data("page") || 1;
	this.pages = this.element.data("pages") || 1;
	this.page_size = this.element.data("page_size") || 10;
	var callback = this.element.data("callback");
	if (callback) {
		callback = callback.split(".");
		this.data.class = callback[0];
		this.data.method = callback[1];
	}
	this.footer = this.element.data("footer") || "";
	var instance = this;
	this.update = function() {
		this.page = Math.max(Math.min(this.page, this.pages), 1);
		if (this.data.class && this.data.method) {
			this.data.page = this.page;
			this.data.page_size = this.page_size;
			$.ajax({
				type : "POST",
				dataType : "json",
				data : this.data,
				success : this.onupdate
			});
		}
	};
	this.onupdate = function(response) {
		var rows = response.hasOwnProperty("rows") ? response.rows : [];
		var row_count = response.hasOwnProperty("count") ? response.count : 0;
		instance.pages = Math.ceil(row_count / instance.page_size);
		instance.update_body(rows);
		instance.update_footer(row_count);
		instance.update_pagination();
	};
	this.update_body = function(rows) {
		var columns = this.element.find("> table th").map(function() {
			return { "type" : $(this).data("type"), "name" : $(this).data("name") };
		});
		var tbody = this.element.find("> table tbody").empty();
		var rowid = this.element.data("rowid");
		rows.forEach(function(row) {
			var tr = $("<tr>").data('id', row[rowid]);
			columns.each(function () {
				var items = this.name.split(',');
				if(items.length > 1) {
					var td = $('<td>');
					for(var i = 0; i < items.length; i++) {
						if (row.hasOwnProperty(items[i]) && instance[this.type]) {
							instance[this.type].apply(td, [ row[items[i]], row ])
						}
					}
					td.appendTo(tr);
				} else {
					if (row.hasOwnProperty(this.name) && instance[this.type]) {
						instance[this.type].apply($("<td>").appendTo(tr), [ row[this.name], row ]);
					} else {
						tr.append($("<td>"));
					}
				}
			});
			tbody.append(tr);
		});
	};
	this.update_footer = function(row_count) {
		this.element.find("> div span").html(
			this.footer
				.replace("%1", Math.min((this.page_size * (this.page - 1)) + 1, row_count))
				.replace("%2", Math.min(this.page_size * this.page, row_count))
				.replace("%3", row_count)
		);
	};
	this.update_pagination = function() {
		this.element.find("> div li").slice(1, -1).remove();
		var next = this.element.find("> div li:last-of-type");
		if (this.pages < 10) {
			for (var i = 1; i <= this.pages; ++i) {
				next.before($("<li>").html(i));
			}
			$(next.siblings()[this.page]).addClass("active");
		} else {
			next.before($("<li>").html(1).addClass((this.page == 1) ? "active" : ""));
			for (var i = Math.max(this.page - 2, 2), end = Math.min(this.page + 2, this.pages - 1); i <= end; ++i) {
				next.before($("<li>").html(i).addClass((this.page == i) ? "active" : ""));
			}
			next.before($("<li>").html(this.pages).addClass((this.page == this.pages) ? "active" : ""));
		}
	};
	this.datetime = function (value) { this.html((new Date(parseInt(value) * 1000)).toLocaleString()); };
	this.date = function (value) { this.html((new Date(parseInt(value) * 1000)).toLocaleDateString()); };
	this.time = function (value) { this.html((new Date(parseInt(value) * 1000)).toLocaleTimeString()); };
	this.string = function (value) { this.html(value); };
	this.minutes = function (value) {
		value = parseInt(value);
		this.html(Math.floor(value / 60) + ":" + ("0" + (value % 60)).slice(-2));
	};
	this.delete = function(value) {
		this.append(
			$("<i>").addClass("btn btn-danger btn-xs glyphicon glyphicon-trash").data("id", value)
		);
	}
}

function show_pass_strength(pass_obj) {

	var data = {};
	data.class = pass_obj.class;
	data.method = pass_obj.method;
	data.value = pass_obj.value;
	if(pass_obj.value.length > 0) {
		$.ajax({
			type : "POST",
			data : data,
			success : function(response) {
				var bar = $(pass_obj.bar_selector);
				if(response) {
					switch(response) {
						case 'weak':{
							bar.css('background', '#DC143C');
							break;
						}
						case 'medium':{
							bar.css('background', '#f7f433');
							break;
						}
						case 'strong':{
							bar.css('background', '#3D9600');
							break;
						}
						default: { bar.css('background', ''); }
					}
				}
			},
			error: function() {
				bar.css('background', '');
			}
		});
	} else {
		var bar = $(pass_obj.bar_selector);
		bar.css('background', '');
	}

}

/**
* gray out an element and it's labels upon checkbox trigger
* @method gray_out
* @param {element} trigger - checkbox trigger element
* @param {element} field   - field to disable when trigger is checked
* @param {element} label   - checkbox label element to disable when trigger is checked
* @param {boole} reverse   - Reverse gray out condition (default true)
*/
function gray_out(trigger, field, label, reverse) {
	$rhs = (reverse == null) ? true : false;
	if(trigger.prop('checked') == $rhs) {
		field.prop('disabled', false);
		field.removeClass('disabled');
		if(label != null) {
			label.removeClass('disabled');
		}
	} else {
		field.prop('disabled', true);
		field.addClass('disabled');
		if(label != null) {
			label.addClass('disabled');
		}
	}
}

function check_dahdi() {
	$.ajax({
		type: "POST",
		url: "/",
		dataType: 'json',
		data: { class: "core", method: 'check_dahdi' },
		success: function(result) {
			if(result == 1) {
				BootstrapDialog.show({
					type: BootstrapDialog.TYPE_WARNING,
					title: ombu.i18n.ombutel.dahdi_reload_title,
					message: ombu.i18n.ombutel.dahdi_reload_content,
					buttons: [{
						label: ombu.i18n.ombutel.dahdi_reload_confirm,
						cssClass: 'btn-warning',
						action: function(dialogItself) {
							do_action("/apply-changes");
							$(this).addClass('hidden');
							dialogItself.close();
						}
					}, {
						label: ombu.i18n.ombutel.dahdi_reload_cancel,
						cssClass: 'btn-primary',
						action: function(dialogItself) {
							dialogItself.close();
						}
					}]
				})
			} else {
				do_action("/apply-changes");
				$(this).addClass('hidden');
			}
		},
		error: function (jqXHR, exception) { //reload regardless
			do_action("/apply-changes");
			$(this).addClass('hidden');
		}
	});
}

function exceed_size(title, text) {
	BootstrapDialog.show({
		type: BootstrapDialog.TYPE_WARNING,
		title: title,
		message: text,
		buttons: [{
			label: ombu.i18n.ombutel.confirm,
			cssClass: 'btn-primary',
			action: function(dialogItself) {
				dialogItself.close();
			}
		}]
	});
	return;
}

function register_timer(module, name, callback, interval, immediate) {
	unregister_timer(module, name);
	if (immediate === undefined) {
		immediate = false;
	}
	if (!ombu.timers[module]) {
		ombu.timers[module] = {};
	}
	if (immediate) {
		callback();
	}
	ombu.timers[module][name] = setInterval(callback, interval);
}

function unregister_timer(module, name) {
	if (ombu.timers[module] && ombu.timers[module][name]) {
		clearInterval(ombu.timers[module][name]);
	}
}

ombu.datatable = function(container, module, method, editable, on_update) {
	var that = this;
	that.offset = that.matched = that.total = that.shown = 0;
	that.max = (function() {
		return $("<select class='page_size'/>").append(
			$("<option/>").val(10).text(10),
			$("<option/>").val(25).text(25),
			$("<option/>").val(50).text(50),
			$("<option/>").val(100).text(100)
		).val(10).change(function() { that.update(); });
	})();
	that.search = (function() {
		return $("<input/>")
			.prop("type", "text")
			.prop("placeholder", ombu.i18n.ombutel.search)
			.on("input", function() { that.offset = 0; that.update(); });
	})();
	that.table = (function() {
		var tfoot = $("<tfoot/>").append($("<tr/>").append($("<td/>").append($("<ul/>"), $("<span/>"))));
		var table = $("<table/>").addClass("table datatable").append($("<thead/>"), $("<tbody/>"), tfoot);
		if (editable) { table.addClass("editable"); }
		table.on("click", "li:first-of-type", function() { that.update(that.offset - that.max.val()); })
		.on("click", "li:last-of-type", function() { that.update(that.offset + that.max.val()); })
		.on("click", "li:not(:first-of-type):not(:last-of-type)", function(event) {
			that.update(($(event.target).data("page") - 1) * that.max.val()); });
		return table;
	})();
	container.append(
		$("<div/>").addClass("datatable").append(
			$("<label/>").text(ombu.i18n.ombutel.rows_per_page).append(that.max),
			that.search
		),
		that.table
	);
	that.update = function(new_offset) {
		if (new_offset != undefined) { that.offset = new_offset; }
		var url = "/?class="+module+
			"&data="+container.data('id')+
			"&method="+method+
			"&offset="+that.offset+
			"&max="+that.max.val()+
			"&search="+that.search.val();
		var request = $.ajax({"url": url, "dataType": "json"}).done(
			function(response) { // update state
				that.offset = response.offset ?? 0;
				that.matched = response.matched ?? 0;
				that.total = response.total ?? 0;
				that.shown = (response.rows ?? []).length;
			},
			function(response) { // populate headers
				var have_buttons = editable || response.buttons.length > 0;
				that.table.find("tfoot td").attr("colspan", Object.keys(response.headers ?? []).length + have_buttons);
				if(jQuery.type(response.headers ?? []) == 'array') { // means that headers do not include tooltips
					var headers = $("<tr/>").append($.map(response.headers ?? [], (header) => {
						return $("<th/>").text(header)
					}));
				} else {
					var headers = $("<tr/>").append($.map(response.headers ?? [], (tooltip, header) => {
						return $("<th/>").text(header)
							.popover({
								title: header,
								trigger: 'hover',
								container: 'body',
								content: tooltip,
								placement: 'top',
							});
					}));
				}
				if (have_buttons) { headers.append("<th/>"); }
				that.table.find("thead").empty().append(headers);
			},
			function(response) { // populate rows
				var have_buttons = editable || (response.buttons ?? []).length > 0;
				that.table.find("tbody").empty().append((response.rows ?? []).map(
					function(row) {
						var row = $("<tr/>").data("key",row.shift()).append(row.map(
							function(value) {
								return $("<td/>").text(value);
							}
						));
						if (have_buttons) {
							var cell = $("<td/>");
							if (editable) {
								var edit_button = $("<button/>").addClass("btn btn-warning btn-xs")
									.append($("<i/>").addClass("glyphicon glyphicon-pencil"));
								var delete_button = $("<button/>").addClass("btn btn-danger btn-xs")
									.append($("<i/>").addClass("fa fa-trash-o"));
								cell.append(edit_button, delete_button);
							}
							if ((response.buttons ?? []).length > 0) {
								cell.append(response.buttons);
							}
							row.append(cell);
						}
						return row;
					}
				));
			},
			function() { // populate statistics
				var first = (that.shown > 0) ? that.offset + 1 : that.offset;
				that.table.find("tfoot span").text(
					ombu.i18n.ombutel.paging
						.replace("#0", first)
						.replace("#1", that.offset + that.shown)
						.replace("#2", that.matched)
						.replace("#3", that.total)
				);
			},
			function() { // populate pagination
				var max = that.max.val();
				var pages = Math.ceil(that.matched / max);
				var current = Math.floor(that.offset / max) + 1;
				var start = Math.max(current - 2, 2);
				var end = Math.min(current + 2, pages - 1);
				var sequence = (pages < 10) ? [].range(1, pages) : [].concat(1, [].range(start, end), pages);
				that.table.find("tfoot ul").empty()
					.append($("<li/>").text(ombu.i18n.ombutel.previous))
					.append(sequence.map(function(i) { return $("<li/>").text(i).data("page", i)
						.addClass((current == i) ? "active" : ""); }))
					.append($("<li/>").text(ombu.i18n.ombutel.next));
			}
		);
		if (on_update) { request.done(on_update); }
	};
	that.update();
};

$(function() {
	$.datepicker.setDefaults({
		dateFormat: ombu.picker_dateformat,
		timeformat: ombu.picker_timeformat,
		changeMonth: true,
		changeYear: true
	});
	$.timepicker.setDefaults({
		timeformat: ombu.picker_timeformat
	});

	$.ajaxSetup({ cache: true });
	$(document).ajaxStart(function() {
		NProgress.configure({
			speed: 200,
			minimum: 0.08,
			trickleRate: 0.02,
			trickleSpeed: 800,
			showSpinner: false
		});
		NProgress.start();
	}).ajaxStop(function() {
		NProgress.done();
	});
	$("#pbx-content").on("change", ".imageupload input[type='file']", function() {
		var parent = $(this).closest(".imageupload");
		if (window.FileReader) {
			if (this.files instanceof FileList && this.files.length > 0) {
				var reader = new FileReader();
				reader.onload = function() {
					parent.find(".preview img:not(:last-of-type)").remove();
					parent.find(".preview").prepend(
						$("<img>").attr("src", this.result)
					);
					parent.removeClass("default");
				};
				reader.readAsDataURL(this.files[0]);
			}
		} else {
			parent.removeClass("default");
		}
	});
	$("#pbx-content").on("click", ".imageupload .remove", function() {
		var parent = $(this).closest(".imageupload");
		parent.find("input[type='file'],input[type='hidden']").each(function() { this.value = ""; });
		parent.find(".preview img:not(:last-of-type)").remove();
		parent.addClass("default");
	});
	$("#pbx-content").on("change", ".fileupload input[type='file']", function() {
		$(this).closest(".fileupload").find(".form-control").html(this.value);
	});

	$(document).on("click", ".modal-dialog .formset, #pbx-content .formset", function() {
		const field = $(this).closest(".form-group");
		function sort_options(values) {
			// sort options, selected first as defined by user
			const options = field.find("select option").toArray();
			const selected_options = values.map((value) => options.find((option) => option.value == value));
			const remaining_options = options.filter((option) => !values.includes(option.value));
			remaining_options.sort(((a, b) => $(a).text().localeCompare($(b).text())));
			field.find("select").empty().append(selected_options, remaining_options);
		}
		sort_options(field.find("select").val() ?? []);
		multiselect_modal(
			field.find(".control-label").text(),
			field.find("select"),
			function() {
				// Getting the selected values takes five lines because:
				// - On add, order of options is not updated.
				// - On remove, li.ui-element become hidden so need to filter them out.
				// - Value itself not present on li.ui-elements, need to dig into plugin internals.
				const values = $(this).find(".list.selected li.ui-element").filter((idx, elem) => {
					return $(elem).data("multiselect.optionLink").prop("selected");
				}).map((idx, elem) => {
					return $(elem).data("multiselect.optionLink").val();
				}).toArray();
				const labels = values.map((value) => {
					return field.find(`select option[value='${value}']`).text();
				});
				sort_options(values);
				field.find(".form-control").text(labels);
				field.find("select").val(values);
			}
		);
		return false;
	});
	$("#pbx-content").on("click", ".recording.input-group-addon", function () {
		var element = $(this);
		var recording = element.closest(".tab-content").find("#" + element.data("ref")).val();
		if (recording !== null && recording.length > 0) {
			if (recording == "1") { recording = element.data("default"); }
			pbx_request("recmanagement", "play", "play", recording, null, false);
		}
	});
	$("#pbx-content").on("click", ".pagedtable > div li", function () {
		var table = $(this).closest(".pagedtable");
		var instance = table.prop("instance");
		if (instance) {
			var index = $(this).index();
			if (index == 0) {
				instance.page--;
			} else if (index == $(this).parent().children().last().index()) {
				instance.page++;
			} else {
				instance.page = $(this).html();
			}
			instance.update();
		}
	});

	//kill the audio when modal is closed
	$(document).on("click", ".sound_player_close, .btn-primary", function() {
		$.each($(this).closest('div.modal-dialog').find('audio'), function () {
			this.pause();
		});
	});

	helpPopover(); // for login screen I guess
	$(document).on("show.bs.popover", function () { // kill any existing tooltips before displaying another
		$("div[id^=popover][role=tooltip].popover").remove();
	});

	$(document).on("click", "#about-ombutel button.show_service_agreement", function() {
		pbx_request("license");
	});

	$("#pbxlogin-form").on("keyup", ".form-group-admin_pass input#admin_pass, \
		.form-group-admin_pass_retype input#admin_pass_retype", function() {
		show_pass_strength({
			"class" : "core",
			"method" : "show_strength",
			"value" : this.value,
			"bar_selector" : "#pbxlogin-form input#"+ this.name +" + .password_strength"
		});
	});

	$(document).on("keyup", "div#profile-modal form.profile-modal input#password, \
		div#profile-modal form.profile-modal input#password_confirm", function() {
		show_pass_strength({
			"class" : "core",
			"method" : "show_strength",
			"value" : this.value,
			"bar_selector" : "div#profile-modal form.profile-modal input#"+ this.name +" + .password_strength"
		});
	});
});

var mobile = false;
var asideNav = $("#aside-nav");
var navBar = $(".ombutelPBX .navbar");
var contentArea = $("#modules-content-area");
var mobileLogo = $(".mobile-logo");
var global_search_container = $("#global-search-container");
var navBackDrop = $("#aside-backdrop");

$(function () {
	toastr.options = {
		positionClass: "toast-top-center",
		closeButton: true,
		newestOnTop: true,
		preventDuplicates: true,
		onShown: function() { $(this).trigger("toastr.shown"); },
		onHidden: () => { $(document).trigger("toastr.hidden"); },
	};

	$.shake({
		violence: 3.0,
		debounce: 1000,
		callback: toggleAside
	});

	$("#aside-navigation").find("a").click(function (e) {
		e.preventDefault();
		$(this).tab('show');
	});

	$("ul.children-menu li").find("a").click(function(e) {
		if(!$(this).hasClass('link')) {
			e.preventDefault();
		}
		var checkElement = $(this).next();
		if ($(this).parent('li').parent('ul').hasClass('sub-menu')) {
			//We are on third level we don't need collapse anything
			return true;
		}

		$("ul.children-menu li").removeClass('active');
		$("ul.children-menu li a").removeClass('sub-menu-indicator-minus');
		$(this).closest('li').addClass('active');
		$(this).addClass('sub-menu-indicator-minus');
		if ((checkElement.is('ul')) && (checkElement.is(':visible'))) {
			$(this).closest('li').removeClass('active');
			$(this).removeClass('sub-menu-indicator-minus');
			checkElement.slideUp('normal', function () {
				$(this).trigger("slideUp.complete");
			});
		}

		if ((checkElement.is('ul')) && (!checkElement.is(':visible'))) {
			$('ul.children-menu ul:visible').slideUp('normal', function () {
				$(this).trigger("slideUp.complete");
			});
			checkElement.slideDown('normal', function () {
				$(this).trigger("slideDown.complete");
			});
		}

		if (checkElement.is('ul')) {
			return false;
		} else {
			return true;
		}
	});

	var scrollSettings = {
		wheelSpeed: 2,
		wheelPropagation: true,
		minScrollbarLength: 20
	};

	$("#mobile-search").click(function() {
		global_search_container.toggleClass('hidden-xs');
		global_search_container.find('#ombutel-search').focus();
	});

	$("#close-global-search").click(function() {
		hideMainSearch();
	});

	$("#openAside").click(function () {
		toggleAside();
	});

	navBackDrop.click(function () {
		toggleAside();
	});
});

async function check_backup_restore_status() {
	clear = () => { $.ajax("/clear-backup-restore-status"); };
	while (true) {
		try {
			const response = JSON.parse(await $.ajax("/backup-restore-status", { dataType: "text" }));
			if (response) {
				if (response.state == "success" && response.action == "reload") {
					window.location.reload();
				}
				switch (response.status) {
					case "idle": return;
					case "working":
						// Don't tell user things are still in progress because its not reliable enough for Eran
						break;
					case "done":
						return toastr.success(response.message, response.title,
							{ timeOut: 0, extendedTimeOut: 0, onHidden: clear });
					case "error": {
						return toastr.error(response.message, response.title,
							{ timeOut: 0, extendedTimeOut: 0, tapToDismiss: false, onHidden: clear });
					}
				}
			}
		} catch (err) {
			if (err.status == 403) {
				return; // not allowed, stop asking
			}
		}
		await new Promise((resolve) => setTimeout(resolve, 5000));
	}
}

function toggleAside() {
	$("#aside-nav").toggle({
		effect: 'slide',
		easing: 'swing',
		duration: '200',
		complete: function () {
			asideNav.toggleClass('closeMenu');
			if (!mobile) {
				navBar.toggleClass('closeMenu');
				contentArea.toggleClass('closeMenu');
			} else {
				if (navBackDrop.is(":visible")) {
					navBackDrop.hide();
				} else {
					navBackDrop.show();
				}
			}

			if (!asideNav.is(":visible")) {
				navBackDrop.hide();
			}

			mobileLogoInteraction();
		}
	});
}

function mobileLogoInteraction() {
	if (bodyWidth <= 425) {
		mobileLogo.addClass('xtra-small');
		mobileLogo.find('img').addClass('size-logo-small').removeClass('size-logo');
	} else {
		mobileLogo.removeClass('xtra-small');
		mobileLogo.find('img').addClass('size-logo').removeClass('size-logo-small');
	}

	if (asideNav.is(":visible")) {
		mobileLogo.hide();
	} else {
		mobileLogo.show();
	}

}

function mobileInteraction() {
	if (bodyWidth <= 992) {
		mobile = true;
		asideNav.addClass('closeMenu');
		navBar.addClass('mobile');
		contentArea.addClass('mobile');
		navBackDrop.hide();
		mobileLogoInteraction();
	} else {
		mobile = false;
		asideNav.removeClass('closeMenu');
		navBar.removeClass('mobile closeMenu');
		contentArea.removeClass('mobile closeMenu');
		navBackDrop.hide();
		mobileLogoInteraction();
	}
}

function themeDelegate() {
	//~ $("ul.module-tabs").tabdrop({
		//~ text: ombu.i18n.ombutel.more
	//~ });

	hideMainSearch();
}

function hideMainSearch(){
	global_search_container.addClass('hidden-xs');
}

async function oidc_login(params) {
	const [client_id, client_secret, redirect_uri, authorization, token] = params;
	const client = new jso.JSO({
		client_id,
		redirect_uri,
		authorization,
		scopes: { request: ["openid", "profile"]},
		response_type: "code",
		client_secret,
		token,
		debug: true,
	});
	await client.getToken({
		scopes: {
			request: ["openid", "profile"],
		}
	});
}

$("body").on("change", "select.form-control.dependent[data-child]", function(event) {
	var element = $(event.target);
	var category = element.val();
	var child = element.closest('form').find("[name='"+element.data("child")+"']");
	var url = "/?class=core&method=getDestinationChildOptions&category="+category;
	$.ajax(url).done(function(options) {
		child.empty().append(options.map(function(option) {
			return $("<option/>").html(option.content).val(option.value);
		})).find(":nth-child(2)").prop("selected", true);
		child.trigger("change");
	});
});

$("#ombutel-modules-content").on("click", "button.btn-menu-mod", async (event) => {
	const tab = (isMultiTab) ? $(event.currentTarget).closest(".tab-pane") : modulesContent;
	const title = tab.data("menu_title");
	const module_name = tab.data("module");
	$("#aside-nav-right h4").text(title);
	showRightMenu();
	try {
		await pbx_request(module_name, "menu", "menu");
	} finally {
		$("#module-menu-items").removeClass("spinning").find(".sk-spinner").remove();
	}
});
